@@ -28,14 +28,19 @@ only_A_estimate <- function(net_object                             ,
       net_type       <- net_stat$net_type
 
       data_cv        <- .CreateDataCV_onlyA(net_object, deg_thresh = net_stat$deg_thresh, p = p)
+      #print("before applying CV")
       cv_result      <- .OnlyA_CV(data_cv, stop_cond = stop_cond * 10, mode_reg_A = mode_reg_A,...)
+      #print("after applying CV")
+      #print(cv_result$alpha_optimal)
       # find a rough estimate of the attachment function based on the model Ak = k^alpha
-      result_temp  <- PAFit(net_stat, 
-                            mode_f      = "Log_linear"            , 
-                            only_PA     = TRUE                    ,
-                            alpha_start = cv_result$alpha_optimal , 
-                            stop_cond   = stop_cond * 10          , # loose convergence condition 
-                            ...) 
+      # result_temp  <- PAFit(net_stat, 
+      #                       mode_f      = "Log_linear"            , 
+      #                       only_PA     = TRUE                    ,
+      #                       alpha_start = cv_result$alpha_optimal , 
+      #                       stop_cond   = stop_cond * 10          , # loose convergence condition 
+      #                       ...)
+      # #print("after parametric PAFit")
+      #print(cv_result$alpha_optimal)
       # feed the estimated attachment function for a warm-start re-run with nonparametric attachment function
       result  <- PAFit(net_stat, 
                    r           = cv_result$r_optimal     , 
